/* libiir/src/libiir/200_iir.c
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/* struct iir_coeff_t
 *  Holds a general IIR filter (i.e. the set of coefficients that define it).
 *  nc >= 1 and nd >= 1.
 */
struct iir_coeff_t {
    int nc, nd;
    double* c, * d;
};



/* iir_coeff_new()
 *  Allocates a new set of coefficient objects.
 */
struct iir_coeff_t*
iir_coeff_new(int nc, double* c, int nd, double* d)
{
    struct iir_coeff_t* coeff;

    if(nc < 1 || nd < 1) {
        errno = EINVAL;
        return 0;
    }

    coeff = malloc(sizeof(struct iir_coeff_t));
    coeff->nc = nc;
    coeff->nd = nd;
    coeff->c = malloc(sizeof(double) * nc);
    coeff->d = malloc(sizeof(double) * nd);
    memcpy(coeff->c, c, sizeof(double) * nc);
    memcpy(coeff->d, d, sizeof(double) * nd);

    return coeff;
}



/* iir_coeff_free()
 *  Frees memory associated with ‘coeff’.
 */
void
iir_coeff_free(struct iir_coeff_t* coeff)
{
    if(!coeff) return;
    free(coeff->c);
    free(coeff->d);
    free(coeff);
}



/* struct iir_filter_t
 *  An instantiated IIR filter. This is actually a linked list node, so that we
 *  can create chains of filters. It also has a copy of the coefficients so that
 *  the library user doesn't need to keep the struct iir_coeff_t instances
 *  around.
 */
struct iir_filter_t {
    /* pointer to next stage */
    struct iir_filter_t* next;

    /* coefficients for this stage */
    int nc, nd;
    double* c, * d;

    /* state for this stage */
    int ready; /* if clear, first sample is used to set initial conditions */
    double* x, * y;
    int xpos, ypos;
};



/* iir_filter_new()
 *  Allocates a new IIR filter instance object, copying the coefficients out of
 *  ‘coeff’.
 */
struct iir_filter_t*
iir_filter_new(const struct iir_coeff_t* coeff)
{
    struct iir_filter_t* fi;

    fi = malloc(sizeof(struct iir_filter_t));
    fi->next = 0;
    fi->ready = fi->xpos = fi->ypos = 0;

    /* copy in the coefficients */
    fi->nc = coeff->nc;
    fi->nd = coeff->nd;
    fi->c = malloc(sizeof(double) * fi->nc);
    fi->d = malloc(sizeof(double) * fi->nd);
    memcpy(fi->c, coeff->c, sizeof(double) * fi->nc);
    memcpy(fi->d, coeff->d, sizeof(double) * fi->nd);

    /* allocate space for state */
    fi->x = malloc(sizeof(double) * fi->nc);
    fi->y = malloc(sizeof(double) * fi->nd);

    return fi;
}



/* iir_filter_free()
 *  Frees a filter chain.
 */
void
iir_filter_free(struct iir_filter_t* fi)
{
    struct iir_filter_t* next;

    while(fi) {
        next = fi->next;
        free(fi->c);
        free(fi->d);
        free(fi->x);
        free(fi->y);
        free(fi);
        fi = next;
    }
}



/* iir_filter_chain()
 *  Extends an IIR filter instance with another filter.
 */
void
iir_filter_chain(struct iir_filter_t* fi, const struct iir_coeff_t* coeff)
{
    /* go to end of linked list */
    while(fi->next) fi = fi->next;

    /* add to end of chain */
    fi->next = iir_filter_new(coeff);
}



/* iir_filter_copy()
 *  Performs a deep copy of a filter instance chain.
 */
struct iir_filter_t*
iir_filter_copy(const struct iir_filter_t* fi, int state)
{
    struct iir_filter_t* head = 0, * tail = 0, * copy;

    while(fi) {
        copy = malloc(sizeof(struct iir_filter_t));
        copy->next = 0;
        copy->nc = fi->nc;
        copy->nd = fi->nd;
        copy->c = malloc(sizeof(double) * fi->nc);
        copy->x = malloc(sizeof(double) * fi->nc);
        copy->d = malloc(sizeof(double) * fi->nd);
        copy->y = malloc(sizeof(double) * fi->nd);
        memcpy(copy->c, fi->c, sizeof(double) * fi->nc);
        memcpy(copy->d, fi->d, sizeof(double) * fi->nd);

        if(state && fi->ready) {
            copy->ready = 1;
            memcpy(copy->x, fi->x, sizeof(double) * fi->nc);
            memcpy(copy->y, fi->y, sizeof(double) * fi->nd);
            copy->xpos = fi->xpos;
            copy->ypos = fi->ypos;
        } else {
            copy->ready = copy->xpos = copy->ypos = 0;
        }

        if(!head) {
            head = tail = copy;
        } else {
            tail->next = copy;
            tail = copy;
        }

        fi = fi->next;
    }

    return head;
}



/* iir_filter()
 *  Processes a sample, possibly dispatching it down the chain.
 */
static double
iir_get_xy(const double* xy, int pos, int max, int step)
{
    pos -= step + 1;
    if(pos < 0) pos += max;
    return xy[pos];
}

static double
iir_dc_gain(const struct iir_filter_t* fi)
{
    int i;
    double sum, gain;

    /* If the forward path DC gain is less than a threshold then we assume ‘fi’
     * has high pass characteristics.
     */
    sum = 0;
    for(i = 0; i < fi->nc; ++i) sum += fi->c[i];
    gain = sum;

    sum = 0;
    for(i = 0; i < fi->nd; ++i) sum += fi->d[i];
    if(fabs(sum - 1) < 0.001) {
        /* ergh, looks unstable at DC; pretend DC gain is 0 */
        return 0;
    }

    gain /= (1 + sum);
    return gain;
}

double
iir_filter(struct iir_filter_t* fi, double samp)
{
    int i;
    double gain;

    while(fi) {
        if(!fi->ready) {
            /* initial conditions */
            gain = iir_dc_gain(fi);
            for(i = 0; i < fi->nc; ++i) fi->x[i] = samp;
            for(i = 0; i < fi->nd; ++i) fi->y[i] = samp * gain;
            fi->ready = 1;
        }

        /* update input array with sample x(t) */
        fi->x[fi->xpos] = samp;
        if(++fi->xpos == fi->nc) fi->xpos = 0;
        samp = 0;

        /* sum of c[i].x(t-i) */
        for(i = 0; i < fi->nc; ++i) {
            samp += fi->c[i] * iir_get_xy(fi->x, fi->xpos, fi->nc, i);
        }

        /* sum of d[i].y(t-i-1) */
        for(i = 0; i < fi->nd; ++i) {
            samp -= fi->d[i] * iir_get_xy(fi->y, fi->ypos, fi->nd, i);
        }

        /* update output array with new result y(t) */
        fi->y[fi->ypos] = samp;
        if(++fi->ypos == fi->nd) fi->ypos = 0;

        fi = fi->next;
    }

    return samp;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
