/* libiir/src/libiir/200_iir.h
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup iir Basic IIR filtering

The functions in this module present a basic interface for representing IIR
filters, creating instances of (possibly chained) IIR filters, and filtering an
input sample.

A general IIR filter consists of a set of coefficients, and may be created
through \ref iir_coeff_new(). The filter object (the opaque <code>struct
iir_coeff_t</code>) is then used to instantiate specific filters (the opaque
<code>struct iir_filter_t</code>) through \ref iir_filter_new(). Each filter
instance may have an arbitrary further number of IIR filters chained on to it
through \ref iir_filter_chain(). The filter processes one sample at a time
through \ref iir_filter().

*/
/*!@{*/



/* opaque structure */
struct iir_coeff_t;



/*! \brief Create general IIR filter

\param nc Number of \a c coefficients.
\param c Array of \a c coefficients.
\param nd Number of \a d coefficients.
\param d Array of \a d coefficients.
\returns Pointer to new general IIR filter object.

This function creates a new general IIR filter object which may be used to
create filter instances through \ref iir_filter_new() or chained on to existing
instances through \ref iir_filter().

See \ref iir_structure for a full explanation of the parameters.

*/
struct iir_coeff_t* iir_coeff_new(int nc, double* c, int nd, double* d)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Free general IIR filter

\param coeff Pointer to IIR filter object. May be 0.

Frees a set of IIR filter coefficients previously allocated through
\ref iir_coeff_new(). Can be called on a null pointer without consequences.
Note that \ref iir_filter_new() and \ref iir_filter_chain() actually store a
copy of the coefficients, so it is possible to free \a coeff even if existing
filters are still using its coefficient values.

*/
void iir_coeff_free(struct iir_coeff_t* coeff);



/* opaque structure */
struct iir_filter_t;



/*! \brief Create IIR filter instance

\param coeff Filter coefficients to use.
\returns Pointer to new instance of IIR filter.

Creates a new instance of a general IIR filter. The set of coefficients \a coeff
is copied into the returned structure, meaning the coefficients can be freed
after this function returns if they will not be needed again.

The first sample passed through \ref iir_filter() will be used to set initial
conditions (see \ref iir_structure). 

An arbitrary number of further filters may be chained on to the end of this
instance through \ref iir_filter_chain().

*/
struct iir_filter_t* iir_filter_new(const struct iir_coeff_t* coeff)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Free IIR filter instance

\param fi Filter object to free. May be 0.

Frees a previously-allocated IIR filter instance. Can be called on a null
pointer without consequences.

*/
void iir_filter_free(struct iir_filter_t* fi);



/*! \brief Add a further IIR filter to a filter instance

\param fi Filter instance to chain onto.
\param coeff New IIR filter coefficients to add to chain.

Extends an existing IIR filter by chaining a new set of coefficients onto the
end. This can be used for &gt;4th order Butterworth filters, for example. This
copies the set of coefficients from \a coeff so the coefficients can be freed
after this function returns if they are no longer required.

*/
void iir_filter_chain(struct iir_filter_t* fi, const struct iir_coeff_t* coeff)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief Create a deep copy of an IIR filter instance

\param fi Filter instance to copy.
\param state Non-zero to copy state as well.
\returns Pointer to newly-allocated filter instance.

Performs a deep copy of the filter instance \a fi. If \a state is non-zero,
then the internal state of \a fi is copied as well (otherwise it is as treated
as a brand new instance).

*/
struct iir_filter_t* iir_filter_copy(const struct iir_filter_t* fi, int state)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Process a sample

\param fi Filter object to run.
\param samp Input sample \c x(t).
\returns Filtered output sample \c y(t).

Given the input sample \c x(t) (the parameter \a samp), runs the filter chain in
\a fi and produces the output sample \c y(t), which it returns.

*/
double iir_filter(struct iir_filter_t* fi, double samp);



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
