/* libiir/src/libiir/300_common_filters.h
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup common_filters Common types of IIR filter

Functions to create coefficients for various common types of IIR filter. The
coefficient structures which are returned may be used to instantiate IIR
filters using \ref iir_filter_new().

The Butterworth filter code comes from the Exstrom Labs LLC code available under
GPLv2 or later and published at http://www.exstrom.com/journal/sigproc/ . There
is a copy of the original code available in the top level of this project.

*/
/*!@{*/



/*! \brief nth-order Butterworth low-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner Corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1)
\returns Newly-allocated IIR filter coefficients.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type low pass filter with gain \a gain and corner
frequency \a corner.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequency \a corner is expressed as a fraction of the sampling
frequency (which is of course not known by the IIR code). It should lie between
0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency).

*/
struct iir_coeff_t* iir_butterworth_lowpass(int order,
    double gain,
    double corner)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief nth-order Butterworth high-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param corner Corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a corner ≤ 1)
\returns Newly-allocated IIR filter coefficients.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type high pass filter with gain \a gain and corner
frequency \a corner.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequency \a corner is expressed as a fraction of the sampling
frequency (which is of course not known by the IIR code). It should lie between
0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency).

*/
struct iir_coeff_t* iir_butterworth_highpass(int order,
    double gain,
    double corner)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief nth-order Butterworth band-pass

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param c1 Low corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a c1 ≤ 1)
\param c2 High corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a c2 ≤ 1, and \a c1 &lt; \a c2)
\returns Newly-allocated IIR filter coefficients.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type band pass filter with gain \a gain and corner
frequencies \a c1 and \a c2.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequencies \a c1 and \a c2 are expressed as a fraction of the
sampling frequency (which is of course not known by the IIR code). They should
lie between 0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency),
and \a c2 should be greater than \a c1.

*/
struct iir_coeff_t* iir_butterworth_bandpass(int order,
    double gain,
    double c1,
    double c2)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief nth-order Butterworth band-stop

\param order Order of filter (≥1).
\param gain Linear gain of filter.
\param c1 Low corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a c1 ≤ 1)
\param c2 High corner frequency expressed as a fraction of Nyquist
    (0 ≤ \a c2 ≤ 1, and \a c1 &lt; \a c2)
\returns Newly-allocated IIR filter coefficients.

Uses the Exstrom labs code to compute the coefficients of an nth-order (param
\a order) Butterworth-type band stop filter with gain \a gain and corner
frequencies \a c1 and \a c2.

Note it is recommended to chain multiple filters together to build anything
greater than a 4th-order filter. This function won't do that directly for you.
\a gain will usually be set to be 1.0.

The corner frequencies \a c1 and \a c2 are expressed as a fraction of the
sampling frequency (which is of course not known by the IIR code). They should
lie between 0 (0Hz) and 1 (the Nyquist frequency, or ½ the sampling frequency),
and \a c2 should be greater than \a c1.

*/
struct iir_coeff_t* iir_butterworth_bandstop(int order,
    double gain,
    double c1,
    double c2)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
