/* libiir/src/libiir/300_common_filters/100_butterworth.c
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/* Each of the functions below is an impedance-matching layer between the
 * Exstrom code (in 000_exstrom_butterworth.c) and libiir. Although there is a
 * fair bit of code duplication, there are just enough subtle differences in the
 * interface and implementation of the Exstrom code to make abstracting these
 * four functions into one more effort than it is worth.
 */



struct iir_coeff_t* iir_butterworth_lowpass(int order,
    double gain,
    double corner)
{
    int i, nc, nd, * ci;
    double* d, * c;
    struct iir_coeff_t* coeff;

    if(order < 1 || corner < 0 || corner > 1) {
        errno = EINVAL;
        return 0;
    }

    /* get coefficients from Exstrom code */
    d = dcof_bwlp(order, corner);
    nd = order + 1;
    ci = ccof_bwlp(order);
    nc = order + 1;
    gain *= sf_bwlp(order, corner);

    /* compute scaled ‘c’ coefficients */
    c = malloc(sizeof(double) * nc);
    for(i = 0; i < nc; ++i) c[i] = ci[i] * gain;

    /* Instantiate filter structure. Note in Exstrom code that d[0] is always
     * 1.0 and not used; the Güralp code doesn't represent it, hence the shift
     * by 1. */
    coeff = iir_coeff_new(nc, c, nd - 1, d + 1);

    /* clean up */
    free(ci);
    free(c);
    free(d);

    return coeff;
}



struct iir_coeff_t* iir_butterworth_highpass(int order,
    double gain,
    double corner)
{
    int i, nc, nd, * ci;
    double* d, * c;
    struct iir_coeff_t* coeff;

    if(order < 1 || corner < 0 || corner > 1) {
        errno = EINVAL;
        return 0;
    }

    /* get coefficients from Exstrom code */
    d = dcof_bwhp(order, corner);
    nd = order + 1;
    ci = ccof_bwhp(order);
    nc = order + 1;
    gain *= sf_bwhp(order, corner);

    /* compute scaled ‘c’ coefficients */
    c = malloc(sizeof(double) * nc);
    for(i = 0; i < nc; ++i) c[i] = ci[i] * gain;

    /* Instantiate filter structure. Note in Exstrom code that d[0] is always
     * 1.0 and not used; the Güralp code doesn't represent it, hence the shift
     * by 1. */
    coeff = iir_coeff_new(nc, c, nd - 1, d + 1);

    /* clean up */
    free(ci);
    free(c);
    free(d);

    return coeff;
}



struct iir_coeff_t* iir_butterworth_bandpass(int order,
    double gain,
    double c1,
    double c2)
{
    int i, nc, nd, * ci;
    double* d, * c;
    struct iir_coeff_t* coeff;

    if(order < 1 || c1 < 0 || c1 > c2 || c2 > 1) {
        errno = EINVAL;
        return 0;
    }

    /* get coefficients from Exstrom code */
    d = dcof_bwbp(order, c1, c2);
    nd = 2 * order + 1;
    ci = ccof_bwbp(order);
    nc = 2 * order + 1;
    gain *= sf_bwbp(order, c1, c2);

    /* compute scaled ‘c’ coefficients */
    c = malloc(sizeof(double) * nc);
    for(i = 0; i < nc; ++i) c[i] = ci[i] * gain;

    /* Instantiate filter structure. Note in Exstrom code that d[0] is always
     * 1.0 and not used; the Güralp code doesn't represent it, hence the shift
     * by 1. */
    coeff = iir_coeff_new(nc, c, nd - 1, d + 1);

    /* clean up */
    free(ci);
    free(c);
    free(d);

    return coeff;
}



struct iir_coeff_t* iir_butterworth_bandstop(int order,
    double gain,
    double c1,
    double c2)
{
    int i, nc, nd;
    double* d, * c;
    struct iir_coeff_t* coeff;

    if(order < 1 || c1 < 0 || c1 > c2 || c2 > 1) {
        errno = EINVAL;
        return 0;
    }

    /* get coefficients from Exstrom code */
    d = dcof_bwbs(order, c1, c2);
    nd = 2 * order + 1;
    c = ccof_bwbs(order, c1, c2);
    nc = 2 * order + 1;
    gain *= sf_bwbs(order, c1, c2);

    /* compute scaled ‘c’ coefficients */
    for(i = 0; i < nc; ++i) c[i] *= gain;

    /* Instantiate filter structure. Note in Exstrom code that d[0] is always
     * 1.0 and not used; the Güralp code doesn't represent it, hence the shift
     * by 1. */
    coeff = iir_coeff_new(nc, c, nd - 1, d + 1);

    /* clean up */
    free(c);
    free(d);

    return coeff;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
