/* libiir/src/libiir/400_parser.c
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/* IIR_PARSER_CHAIN()
 *  In the parser functions below, we are passed a pointer to the pointer to the
 *  IIR filter structure. The pointed-to pointer must be set if we are parsing
 *  the very first set of coefficients, or chained otherwise. (This structure is
 *  necessary as some of the parser functions result in multiple coefficient
 *  sets). This macro handles that. ‘_fi’ is of type ‘struct iir_filter_t**’.
 */
#define IIR_PARSER_CHAIN(_fi, _coeff) do { \
    if(*_fi) iir_filter_chain(*_fi, _coeff); \
    else *_fi = iir_filter_new(_coeff); \
}while(0)



/* iir_parser_raw()
 *  Parses raw coefficients and adds a filter to the chain ‘fi’. ‘desc’ should
 *  point at the parameter string; a '/' character splits the c and d
 *  coefficients, and a ')' character marks the end of the parameters. The aux
 *  function allocates and parses an array of doubles with each value separated
 *  by a ',' character and the array terminated by an arbitrary character
 *  ‘endc’.
 */
static double*
iir_parser_raw_aux(int* nout, const char** desc, char endc)
{
    int n = 0, sz = 16;
    double* c, x;
    char* endp;

    c = malloc(sizeof(double) * sz);

    while(1) {
        /* parse a single coefficient, taking care with strtod(3) */
        errno = 0;
        endp = 0;
        x = strtod(*desc, &endp);
        if(errno || !endp || endp == *desc) goto fail;
        *desc = endp + 1;

        /* add to array */
        if(n == sz) {
            sz <<= 1;
            c = realloc(c, sizeof(double) * sz);
        }
        c[n++] = x;

        /* check for ',' or end-of-list char */
        if(*endp == ',') continue;
        if(*endp != endc) goto fail;
        break;
    }

    /* done */
    *nout = n;
    return c;

  fail:
    free(c);
    return 0;
}

static int
iir_parser_raw(struct iir_filter_t** fi, const char* desc)
{
    int nc, nd;
    double* c, * d;
    struct iir_coeff_t* coeff;

    /* allocate and parse two arrays of double */
    c = iir_parser_raw_aux(&nc, &desc, '/');
    if(!c) return -1;
    d = iir_parser_raw_aux(&nd, &desc, ')');
    if(!d) {
        free(c);
        return -1;
    }

    /* HACK: rather than calling iir_coeff_new(), save time by not creating a
     * redundant copy and just fill the structure directly */
    coeff = malloc(sizeof(struct iir_coeff_t));
    coeff->nc = nc;
    coeff->nd = nd;
    coeff->c = c;
    coeff->d = d;

    IIR_PARSER_CHAIN(fi, coeff);
    iir_coeff_free(coeff);
    return 0;
}



/* IIR_PARSER_BW_MAX_ORDER
 *  The maximum order in any single Butterworth-type filter. If the given order
 *  exceeds this, we split the resulting filter up into multiple sets of
 *  coefficients of this order or less.
 */
#define IIR_PARSER_BW_MAX_ORDER         (4)



/* iir_parser_bw_aux()
 *  Parses the Butterworth-type parameter string. ‘c2’ may be passed as null if
 *  the filter only has one corner frequency (low or high pass).
 */
static int
iir_parser_bw_aux(const char* desc,
    int* order,
    double* gain,
    double* c1,
    double* c2)
{
    char* endp;

    /* parse order,gain,c1 */
    errno = 0;
    endp = 0;
    *order = strtol(desc, &endp, 0);
    if(errno || !endp || endp == desc || *endp != ',') return -1;
    desc = endp + 1;

    endp = 0;
    *gain = strtod(desc, &endp);
    if(errno || !endp || endp == desc || *endp != ',') return -1;
    desc = endp + 1;

    endp = 0;
    *c1 = strtod(desc, &endp);
    if(errno || !endp || endp == desc) return -1;
    desc = endp + 1;

    /* if c2 is requested, parse that as well */
    if(c2) {
        if(*endp != ',') return -1;
        endp = 0;
        *c2 = strtod(desc, &endp);
        if(errno || !endp || endp == desc) return -1;
    }

    /* check we used the entire parameter string */
    return *endp != ')';
}



/* iir_parser_bw_aux2()
 *  Instantiates a low/high pass type filter with a single corner frequency
 *  using the function ‘bw’. Splits into multiple filters if the order exceeds
 *  the threshold in IIR_PARSER_BW_MAX_ORDER.
 */
static int
iir_parser_bw_aux2(struct iir_filter_t** fi,
    struct iir_coeff_t* (*bw)(int, double, double),
    int order,
    double gain,
    double corner)
{
    struct iir_coeff_t* coeff;

    /* split into segments of 4th order or less */
    if(order >= IIR_PARSER_BW_MAX_ORDER) {
        coeff = bw(IIR_PARSER_BW_MAX_ORDER, gain, corner);
        if(!coeff) return -1;
        while(order >= IIR_PARSER_BW_MAX_ORDER) {
            IIR_PARSER_CHAIN(fi, coeff);
            order -= IIR_PARSER_BW_MAX_ORDER;
        }
        iir_coeff_free(coeff);
        if(!order) return 0;
        /* add a <4th order segment */
    }

    coeff = bw(order, gain, corner);
    if(!coeff) return -1;
    IIR_PARSER_CHAIN(fi, coeff);
    iir_coeff_free(coeff);

    return 0;
}



/* iir_parser_bw_aux3()
 *  Instantiates a band pass/stop type filter with two corner frequencies
 *  using the function ‘bw’. Splits into multiple filters if the order exceeds
 *  the threshold in IIR_PARSER_BW_MAX_ORDER.
 */
static int
iir_parser_bw_aux3(struct iir_filter_t** fi,
    struct iir_coeff_t* (*bw)(int, double, double, double),
    int order,
    double gain,
    double c1,
    double c2)
{
    struct iir_coeff_t* coeff;

    /* split into segments of 4th order or less */
    if(order >= IIR_PARSER_BW_MAX_ORDER) {
        coeff = bw(IIR_PARSER_BW_MAX_ORDER, gain, c1, c2);
        if(!coeff) return -1;
        while(order >= IIR_PARSER_BW_MAX_ORDER) {
            IIR_PARSER_CHAIN(fi, coeff);
            order -= IIR_PARSER_BW_MAX_ORDER;
        }
        iir_coeff_free(coeff);
        if(!order) return 0;
        /* add a <4th order segment */
    }

    coeff = bw(order, gain, c1, c2);
    if(!coeff) return -1;
    IIR_PARSER_CHAIN(fi, coeff);
    iir_coeff_free(coeff);

    return 0;
}



/* iir_parser_bw*()
 *  Various Butterworth-type parsers, built out of the aux blocks above.
 */
static int
iir_parser_bwlp(struct iir_filter_t** fi, const char* desc)
{
    int order;
    double gain, corner;

    /* parse order,gain,corner */
    if(iir_parser_bw_aux(desc, &order, &gain, &corner, 0)) return -1;

    /* instantiate and associate coefficients */
    return iir_parser_bw_aux2(fi, iir_butterworth_lowpass, order, gain, corner);
}

static int
iir_parser_bwhp(struct iir_filter_t** fi, const char* desc)
{
    int order;
    double gain, corner;

    /* parse order,gain,corner */
    if(iir_parser_bw_aux(desc, &order, &gain, &corner, 0)) return -1;

    /* instantiate and associate coefficients */
    return iir_parser_bw_aux2(fi, iir_butterworth_highpass, order, gain, corner);
}

static int
iir_parser_bwbp(struct iir_filter_t** fi, const char* desc)
{
    int order;
    double gain, c1, c2;

    /* parse order,gain,corner */
    if(iir_parser_bw_aux(desc, &order, &gain, &c1, &c2)) return -1;

    /* instantiate and associate coefficients */
    return iir_parser_bw_aux3(fi, iir_butterworth_bandpass, order, gain, c1, c2);
}

static int
iir_parser_bwbs(struct iir_filter_t** fi, const char* desc)
{
    int order;
    double gain, c1, c2;

    /* parse order,gain,corner */
    if(iir_parser_bw_aux(desc, &order, &gain, &c1, &c2)) return -1;

    /* instantiate and associate coefficients */
    return iir_parser_bw_aux3(fi, iir_butterworth_bandstop, order, gain, c1, c2);
}



struct iir_parser_t {
    const char* type;
    int (*parse)(struct iir_filter_t** fi, const char* desc);
};

static struct iir_parser_t iir_parsers[] = {
    {
        .type = "raw",
        .parse = iir_parser_raw,
    },
    {
        .type = "butterworth_lowpass",
        .parse = iir_parser_bwlp,
    },
    {
        .type = "butterworth_highpass",
        .parse = iir_parser_bwhp,
    },
    {
        .type = "butterworth_bandpass",
        .parse = iir_parser_bwbp,
    },
    {
        .type = "butterworth_bandstop",
        .parse = iir_parser_bwbs,
    },
    {
        .type = 0,
    }
};



struct iir_filter_t*
iir_parse(const char* desc)
{
    const char* p, * q;
    int i;
    struct iir_filter_t* fi = 0;

    while(*desc) {
        /* move to next filter in chain */
        if(isspace(*desc)) {
            ++desc;
            continue;
        }

        /* verify type(args) layout */
        p = strchr(desc, '(');
        q = strchr(desc, ')');
        if(!p || !q || q < p) goto fail;

        /* find matching parser function */
        for(i = 0; iir_parsers[i].type; ++i) {
            if((long)strlen(iir_parsers[i].type) != p - desc) continue;
            if(memcmp(iir_parsers[i].type, desc, p - desc)) continue;
            break;
        }
        if(!iir_parsers[i].type) goto fail;

        /* parse description, add to chain */
        if(iir_parsers[i].parse(&fi, p + 1)) goto fail;

        /* consume this filter description from string */
        desc = q + 1;
    }

    /* finish up */
    if(!fi) {
        errno = EINVAL;
        return 0;
    }
    return fi;

  fail:
    iir_filter_free(fi);
    errno = EINVAL;
    return 0;
}



struct iir_filter_t**
iir_parse_n(const char* desc, int n)
{
    struct iir_filter_t* fi, ** a;
    int i;

    if(n < 1) {
        errno = EINVAL;
        return 0;
    }

    fi = iir_parse(desc);
    if(!fi) return 0;

    a = malloc(sizeof(struct iir_filter_t*) * n);
    a[0] = fi;
    for(i = 1; i < n; ++i) a[n] = iir_filter_copy(fi, 0);

    return a;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
