/* libiir/src/libiir/400_parser.h
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



/*! \defgroup parser Parser for user-specified IIR filters

This is a high-level interface that can instantiate a set of IIR filters based
on a user-specified, human-readable string. The intention of this interface is
to allow IIR filters to be specified in configuration files so that they can be
easily modified by the user and easily understood/parsed by the system.

See \ref string_desc for details on the string description format.

*/
/*!@{*/



/*! \brief Instantiate an IIR filter based on a string description

\param desc IIR filter description.
\returns Pointer to newly-allocated IIR filter instance.
\retval 0 on error.

Parses the human-readable description of an IIR filter chain in \a desc,
instantiating an IIR filter object to match. Returns the new filter. If \a desc
cannot be parsed correctly, returns 0 and sets \a errno to \c EINVAL.

*/
struct iir_filter_t* iir_parse(const char* desc)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*! \brief Instantiate a set of IIR filters based on a string description

\param desc IIR filter description.
\param n Number of instances to allocate.
\returns Pointer to array of \a n newly-allocated IIR filter instances.
\retval 0 on error.

Parses the human-readable description of an IIR filter chain in \a desc,
instantiating a set of \a n identical IIR filter objects to match. Returns a
pointer to an array of new filters. If \a desc cannot be parsed correctly,
returns 0 and sets \a errno to \c EINVAL.

The user is responsible for freeing both the array elements (with
\ref iir_filter_free()) and the array itself (with \c free(3)).

*/
struct iir_filter_t** iir_parse_n(const char* desc, int n)
#ifndef DOXYGEN
    __attribute__((malloc,nonnull))
#endif
;



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
