/* libiir/src/tests/plot_filter.c
 * 
 *  Copyright: ©2010, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/

#include "iir.h"

#include <math.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>



#define NPOINTS                         (1000)
#define STEADY_STATE_CYCLES             (20)



char* tmp_fname;
void
unlink_tmpfile(void)
{
    unlink(tmp_fname);
}



int
do_plot(const char* filter_desc, double samp_rat, const char* png_filename)
{
    int fd, ret;
    FILE* fp;
    char cmd_file[] = "/tmp/libiir-plot_filter.cmd.XXXXXX",
        cmd[200];

    fd = mkstemp(cmd_file);
    if(fd == -1) {
        perror("mkstemp");
        return -1;
    }

    fp = fdopen(fd, "w");
    fprintf(fp, "set terminal png size 1000,1000\n"
        "set output '%s'\n"
        "set multiplot layout 2,1 title \"Bode plot for filter '",
        png_filename);
    ret = 0;
    while(*filter_desc) {
        if(isspace(*filter_desc)) {
            if(!ret) {
                ret = 1;
                putc('\n', fp);
            }
        } else {
            putc(*filter_desc, fp);
        }
        ++filter_desc;
    }
    fprintf(fp, "' at %fHz\"\n"
        "set grid\n"
        "set logscale\n"
        "set ytics add ('-3dB' %f)\n"
        "set xlabel 'Frequency (Hz)'\n"
        "set ylabel 'Gain'\n"
        "plot '%s' using 1:2 notitle\n"
        "unset logscale y\n"
        "set yrange [-180:180]\n"
        "set ytics -180,45,180\n"
        "set ylabel 'Phase (degrees)'\n"
        "plot '%s' using 1:3 notitle\n",
        samp_rat, 
        pow(10, -3.0/20),
        tmp_fname,
        tmp_fname);
    fclose(fp);

    snprintf(cmd, sizeof(cmd), "gnuplot %s", cmd_file);
    ret = system(cmd);
    unlink(cmd_file);

    return ret;
}



double
compute_magnitude(double* y, int nsamp)
{
    int samp;
    double max, min;

    max = min = y[0];

    for(samp = 1; samp < nsamp; ++samp) {
        if(y[samp] > max) max = y[samp];
        if(y[samp] < min) min = y[samp];
    }
    return (max - min) / 2;
}



double
compute_phase_deg(double* x, double* y, int nsamp)
{
    int samp, xphase = 0, yphase = 0;
    double xmax, ymax, phase;

    xmax = x[0];
    ymax = y[0];

    for(samp = 1; samp < nsamp; ++samp) {
        if(x[samp] > xmax) {
            xmax = x[samp];
            xphase = samp;
        }

        if(y[samp] > ymax) {
            ymax = y[samp];
            yphase = samp;
        }
    }

    phase = (xphase - yphase) * 360.0 / nsamp;
    if(phase > 180) phase -= 360;
    if(phase <= -180) phase += 360;

    return phase;
}



double
interp(int step, int max, double start, double end)
{
    return start + step * ((end - start) / (max - 1));
}

void
calc_response(FILE* fp,
    struct iir_filter_t* orig_fi,
    double samp_rat,
    double start_freq,
    double end_freq)
{
    int step, samp, cycle_len;
    double freq;
    struct iir_filter_t* fi;
    static double* x = 0, * y = 0;

    for(step = 0; step < NPOINTS; ++step) {
        freq = exp(interp(step, NPOINTS, log(start_freq), log(end_freq)));
        cycle_len = samp_rat / freq + 1;

        /* HACK: allocate persistent buffer; first call must have lowest freq */
        if(!x) {
            x = malloc(sizeof(double) * cycle_len);
            y = malloc(sizeof(double) * cycle_len);
        }

        /* HACK: build steady-state filter response */
        fi = iir_filter_copy(orig_fi, 0);
        for(samp = 0; samp < cycle_len * STEADY_STATE_CYCLES; ++samp) {
            iir_filter(fi, sin(2 * M_PI * freq / samp_rat * samp));
        }

        /* run and record one complete cycle */
        for(samp = 0; samp < cycle_len; ++samp) {
            x[samp] = sin(2 * M_PI * freq / samp_rat *
                (samp + cycle_len * STEADY_STATE_CYCLES));
            y[samp] = iir_filter(fi, x[samp]);
        }

        iir_filter_free(fi);
        fprintf(fp, "%e\t%e\t% 6.2f\n",
            freq,
            compute_magnitude(y, cycle_len),
            compute_phase_deg(x, y, cycle_len));
    }
}



int
safe_strtod(const char* str, double* d)
{
    char* endp = 0;
    errno = 0;
    *d = strtod(str, &endp);
    if(errno || !endp || *endp) return -1;
    return 0;
}



int
main(int argc, char* argv[])
{
    int fd;
    double samp_rat, start_freq, end_freq;
    FILE* fp;
    struct iir_filter_t* fi;

    /* process commandline arguments */
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        fputs("Generates Bode plot for a filter.\n", stdout);
        return 0;
    }

    if(argc != 6) {
        fputs("Usage: plot_filter 'filter_desc' samp_rat start_freq end_freq out.png\n",
            stderr);
        return 1;
    }

    fi = iir_parse(argv[1]);
    if(!fi) {
        fputs("Invalid filter description string.\n", stderr);
        return 1;
    }

    if(safe_strtod(argv[2], &samp_rat) || samp_rat < 1e-6) {
        fputs("Invalid sample rate. Positive float in Hz.\n", stderr);
        return 1;
    }

    if(safe_strtod(argv[3], &start_freq) || start_freq < 1e-6) {
        fputs("Invalid start frequency. Positive float in Hz.\n", stderr);
        return 1;
    }

    if(safe_strtod(argv[4], &end_freq) || end_freq < 1e-6
        || end_freq > samp_rat || end_freq < start_freq)
    {
        fputs("Invalid end frequency. Positive float in Hz, less than sample\n"
            "rate, but greater than start frequency.\n", stderr);
        return 1;
    }

    /* create temporary file for results; gnuplot will use this */
    tmp_fname = strdup("/tmp/libiir-plot_filter.data.XXXXXX");
    fd = mkstemp(tmp_fname);
    if(fd == -1) {
        perror("mkstemp");
        return 1;
    }
    atexit(unlink_tmpfile);

    fp = fdopen(fd, "w");
    calc_response(fp, fi, samp_rat, start_freq, end_freq);
    fclose(fp);

    /* clean up (for valgrind) */
    iir_filter_free(fi);

    /* draw the plot */
    return do_plot(argv[1], samp_rat, argv[5]);
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
